import logging
from contextlib import AsyncExitStack
from types import TracebackType
from typing import Self

import aiohttp

logger = logging.getLogger(__name__)


class NtfyNotifier:
    def __init__(self, ntfy_url: str = 'https://ntfy.homelab.net') -> None:
        self._ntfy_url = ntfy_url
        self._aiohttp_session: aiohttp.ClientSession
        self._async_exit_stack: AsyncExitStack

    async def __aenter__(self) -> Self:
        async with AsyncExitStack() as async_exit_stack:
            self._aiohttp_session = await async_exit_stack.enter_async_context(aiohttp.ClientSession(raise_for_status=True))
            self._async_exit_stack = async_exit_stack.pop_all()
        return self

    async def __aexit__(self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None) -> None:
        await self._async_exit_stack.aclose()

    async def send_notification(self, camera: str, status: bool) -> None:
        logger.debug(f'Sending notification for {camera=}...')
        message = f'{camera} is back online' if status else f'{camera} is offline'
        await self._aiohttp_session.post(self._ntfy_url, json={
            'topic': 'frigate_camera_uptime',
            'title': 'Frigate',
            'message': message,
            'priority': 3,
            'click': f'https://frigate.homelab.net/cameras/{camera}',
            'icon': 'https://cdn.jsdelivr.net/gh/walkxcode/dashboard-icons/png/frigate.png',
        })
        logger.debug(f'Sent notification for {camera=}')
